# dp.oak

dp.oak is a plugin for the [Cycling '74 Max](https://cycling74.com/) development environment
to use your [Luxonis OAK](https://shop.luxonis.com/) sensor on your Windows computer.

### Hardware and Download Requirements 

* [Cycling74 Max requirements](https://cycling74.com/products/max)
* Windows 10 April 2018 (Version 1803, OS Build 17134) release (64-bit) or newer
* [OAK Sensor hardware requirements](https://docs.luxonis.com/projects/hardware/en/latest/index.html)
  vary based on the specific sensor you use.

The Luxonis OAK sensor does not have a shared driver that all applications use. It instead
has all the needed files in the same folder as the plugin. We have split
the downloads into two groups:

* dp.oak files that we create and test. All features
  of dp.oak _except body tracking and AI_ will work with
  only these files. Usually, new features and bugfixes need
  only these files.
* Body tracking neural net files that change less often.
  These download(s) are an unknown size.

### Setup

1. Install [Cycling'74 Max](https://cycling74.com/downloads/) version 6.1.9 or newer (64-bit)
2. Download dp.oak from <https://hidale.com/shop/dp-oak/>
3. [Decompress](https://support.microsoft.com/en-us/help/14200/windows-compress-uncompress-zip-files)
   all the files in this download into an empty directory for dp.oak
4. Next, you must register dp.oak

### Register

1. You can evaluate dp.oak with a two week free trial. If you want to use
   it after trying it, please visit <https://hidale.com/shop/dp-oak/> to purchase a license.
2. Open the .maxhelp file included within the ZIP download.
3. Click the top-left `Register` tab.
4. Type your registration name in the field beside the register button. You
   received your registration name in an email from our store. Hint: your registration
   name is usually _not your email address_.
5. Click the register button and use the dialog box that appears to select
   your dp.oak registration key (.dpreg file)
6. You will see a successful registration in the Max console window

### Body Tracking Setup

The neural net enables body tracking for up to six people. It will run
on the CPU or GPU of your choice using `@skelcompute`.

1. Download the neural net model for body tracking
   * Open the .maxhelp file included within the ZIP download.
     Click on the `body joints` tab. Understand the license and click download.
   * <https://github.com/diablodale/model-zoo/releases/latest/download/movenet_multipose_lightning_1_int32_op15_H160_W256.onnx>
2. Place the model in the same directory as the dp.oak plugin

### Documentation

Documentation and tutorials are at <https://docs.hidale.com/>.

Existing patches for other sensors may work with only a change of the object name to (dp.oak).

### Depth

The depth technology in most OAK sensors is different than many other depth sensors.
It is similar to the stereo vision of humans. The depth will appear more noisy.
Neural nets are usually not affected by this noise.
More info <https://www.google.com/search?q=depth+through+stereo>

### Advanced GPU and Computing Setup

The plugin has settings for compute and GPU devices

* `@opencl` for matrix processing -- default is disabled
* `@skelcompute` for body tracking neural net -- default is primary GPU
* `@transcoder` for color camera decoding -- default is `auto`

We recommend you experiment to discover which settings meet your needs for hardware,
latency, and throughput. You can have significant performance improvements! 🙂

#### OpenCL computation

Some image processing may be done on your GPU or accelerator if your device and drivers
support OpenCL v1.2. Your peformance may be better or worse; depends on specific
hardware and image processing you choose. In our testing, integrated GPUs often perform well.

OpenCL is disabled by default. Enable it by selecting a GPU `@opencl intel` or let Windows
choose the GPU with `@opencl 1`. View all OpenCL devices at the `@opencl` attribute using
an (attrui) or the Max Inspector.

#### Body Tracking computation

`@skelcompute` supports two arguments: compute type and device index.
The default is `directml` and device index `0`. We recommend DirectML.
DirectML works on all modern GPUs and performs the same or better
during testing than other compute types.

```
@skelcompute directml        <- DirectML with first gpu
@skelcompute directml 1      <- DirectML with second gpu
@skelcompute directml intel  <- DirectML with gpu named intel
```

### PoE (Power over Ethernet)

Luxonis PoE (tcp/ip) sensors are now reliable for long-term use.
This long-term reliability can be achieved with two steps

  1. Use plugin v1.3.20240825 or newer
  2. Follow Luxonis' requirement to update OAK PoE sensor factory bootloader
     to v0.0.28 or newer. They recommend using their Device Manager. Luxonis'
     instructions at <https://github.com/luxonis/depthai-core/releases/tag/v2.26.0>
     and <https://docs.luxonis.com/software/depthai-components/bootloader/#Bootloader>.

Several OAK sensors are PoE devices. Instead of connecting with a 3 meter USB cable,
a PoE device is connected with a single ethernet cable and can be in a different building.
Some sensors have IR emitters to work in darkness. Others have weatherproof enclosures.
<https://docs.luxonis.com/projects/hardware/en/latest/index.html#poe-designs>

These devices take a longer time to be detected and to shut down. They also need time
between restarts. It is not unusual to wait 5-8 seconds to find a PoE sensor.
Also, you may need up to 20 seconds after stopping for a PoE sensor to be ready again.
